/*
 * Copyright 2004-2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.hdiv.taglib.html;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.jsp.JspException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.taglib.html.FormTag;
import org.apache.struts.util.RequestUtils;
import org.apache.struts.util.ResponseUtils;
import org.hdiv.dataComposer.IDataComposer;
import org.hdiv.util.HDIVUtil;

/**
 * Renders an HTML <b>&lt;form&gt;</b> element whose contents are described by the
 * body content of this tag. The form implicitly interacts with the specified request
 * scope or session scope bean to populate the input fields with the current property
 * values from the bean. The form bean is located, and created if necessary, based on
 * the form bean specification for the associated ActionMapping.
 * 
 * @author Aritz Rabadan
 * @author Gorka Vicente
 * @see org.apache.struts.taglib.html.FormTag
 */
public class FormTagHDIV extends FormTag {

	/**
	 * Universal version identifier. Deserialization uses this number to ensure that
	 * a loaded class corresponds exactly to a serialized object.
	 */
	private static final long serialVersionUID = -4158917281715942304L;

	/**
     *  Commons Logging instance.
     */
	private static Log log = LogFactory.getLog(FormTagHDIV.class);
	
	private IDataComposer dataComposer;
	
	
    /**
	 * Generates the opening <code>&lt;form&gt;</code> element with appropriate
	 * attributes. It is invoked from doStartTagMethod of FormTag.
	 * 
	 * @see org.apache.struts.taglib.html.FormTag
	 * @see org.hdiv.composer.IDataComposer
	 * @see org.hdiv.util.HDIVUtil#composeAction(String, int)
	 * @since Struts 1.1
	 */
    protected String renderFormStartElement() {    	    	
            
        StringBuffer results = new StringBuffer("<form");
        
        prepareAttribute(results, "name", beanName);
        prepareAttribute(results, "method", getMethod() == null ? "post" : getMethod());        
        prepareAction(results);        
        prepareAttribute(results, "class", styleClass);
        prepareAttribute(results, "enctype", enctype);
        prepareAttribute(results, "onreset", onreset);
        prepareAttribute(results, "onsubmit", onsubmit);
        prepareAttribute(results, "style", style);
        prepareAttribute(results, "id", styleId);
        prepareAttribute(results, "target", target);
  
        results.append(">");
        return results.toString();
    }		
	 
    /**
     * Renders the action attribute
     */
    protected void prepareAction(StringBuffer results) {

    	HttpServletRequest request = (HttpServletRequest) pageContext.getRequest();
    	HttpServletResponse response = (HttpServletResponse) this.pageContext.getResponse();                

        String url = response.encodeURL(RequestUtils.getActionMappingURL(this.action, this.pageContext));
        
        dataComposer = HDIVUtil.getDataComposer();        
        dataComposer.beginRequest(HDIVUtil.getActionMappingName(url));         		
		
        String encodedURL = url;
        
        int questionIndex = url.indexOf("?");
        if (questionIndex >= 0) {
        	encodedURL = HDIVUtil.composeAction(url, questionIndex, "UTF-8");
        } 

        results.append(" action=\"");                
        results.append(encodedURL);
        results.append("\"");                
    }    
    
	/**
	 * Render the end of this form adding HDIV parameter if strategy is cipher or hash.
	 * @exception JspException if a JSP exception has occurred
	 */
	public int doEndTag() throws JspException {
	
		this.addHDIVParameter();	
		return super.doEndTag();
	}
		
	/**
	 * Adds HDIV state as parameter.
	 */
	protected void addHDIVParameter() throws JspException {
		
		String requestId = dataComposer.endRequest();
		
		if (requestId.length() > 0) {
			String hdivParameter = (String) HDIVUtil.getHttpSession().getAttribute("HDIVParameter");
			ResponseUtils.write(pageContext, this.generateHiddenTag(hdivParameter, requestId));
		}		
	}	
	
    /**
	 * Prepares an attribute if the <code>value</code> is not null, appending it to
	 * the the given StringBuffer <code>result</code>.
	 * 
	 * @param result The StringBuffer that output will be appended to.
	 */
	private void prepareAttribute(StringBuffer result, String name, String value) {

		if (value != null) {
        	result.append(" ");
        	result.append(name);
        	result.append("=\"");
        	result.append(value);
        	result.append("\"");			
		}
	}
    
	/**
	 * Renders an HTML <b>&lt;input&gt;</b> element of type hidden.
	 * 
	 * @param name hidden parameter name
	 * @param requestId request identification
	 * @return HTML <b>&lt;input&gt;</b> element of type hidden
	 */
	private String generateHiddenTag(String name, String requestId) {
		
		StringBuffer hdivParameter = new StringBuffer();
		
		hdivParameter.append("<input type=\"hidden\"");		
		this.prepareAttribute(hdivParameter, "name", name);
		this.prepareAttribute(hdivParameter, "value", requestId);
		hdivParameter.append(">\n");
		
		return hdivParameter.toString();
	}

}
